@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2015 Phusion Holding B.V.
+ *  Copyright (c) 2010-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -41,10 +41,13 @@
 #include <Constants.h>
 #include <UnionStationFilterSupport.h>
 
-/* The APR headers must come after the Passenger headers. See Hooks.cpp
+/* The APR/Apache headers must come after the Passenger headers. See Hooks.cpp
  * to learn why.
  */
 #include <apr_strings.h>
+// In Apache < 2.4, this macro was necessary for core_dir_config and other structs
+#define CORE_PRIVATE
+#include <http_core.h>
 
 using namespace Passenger;
 
@@ -228,6 +231,58 @@ passenger_config_merge_dir(apr_pool_t *p, void *basev, void *addv) {
 	return config;
 }
 
+static void
+postprocessDirConfig(server_rec *s, core_dir_config *core_dconf,
+	DirConfig *psg_dconf, bool isTopLevel = false)
+{
+	if (psg_dconf->unionStationSupport == DirConfig::ENABLED) {
+		serverConfig.unionStationSupport = true;
+	}
+}
+
+#ifndef ap_get_core_module_config
+	#define ap_get_core_module_config(s) ap_get_module_config(s, &core_module)
+#endif
+
+void
+passenger_postprocess_config(server_rec *s) {
+	core_server_config *sconf;
+	core_dir_config *core_dconf;
+	DirConfig *psg_dconf;
+	int nelts;
+    ap_conf_vector_t **elts;
+    int i;
+
+	serverConfig.finalize();
+
+	for (; s != NULL; s = s->next) {
+		sconf = (core_server_config *) ap_get_core_module_config(s->module_config);
+		core_dconf = (core_dir_config *) ap_get_core_module_config(s->lookup_defaults);
+		psg_dconf = (DirConfig *) ap_get_module_config(s->lookup_defaults, &passenger_module);
+		postprocessDirConfig(s, core_dconf, psg_dconf, true);
+
+		nelts = sconf->sec_dir->nelts;
+		elts  = (ap_conf_vector_t **) sconf->sec_dir->elts;
+		for (i = 0; i < nelts; ++i) {
+			core_dconf = (core_dir_config *) ap_get_core_module_config(elts[i]);
+			psg_dconf = (DirConfig *) ap_get_module_config(elts[i], &passenger_module);
+			if (core_dconf != NULL && psg_dconf != NULL) {
+				postprocessDirConfig(s, core_dconf, psg_dconf);
+			}
+		}
+
+		nelts = sconf->sec_url->nelts;
+		elts  = (ap_conf_vector_t **) sconf->sec_url->elts;
+		for (i = 0; i < nelts; ++i) {
+			core_dconf = (core_dir_config *) ap_get_core_module_config(elts[i]);
+			psg_dconf = (DirConfig *) ap_get_module_config(elts[i], &passenger_module);
+			if (core_dconf != NULL && psg_dconf != NULL) {
+				postprocessDirConfig(s, core_dconf, psg_dconf);
+			}
+		}
+	}
+}
+
 
 /*************************************************
  * Passenger settings
@@ -237,6 +292,7 @@ DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_root, root)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_default_ruby, defaultRuby)
 DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_log_level, logLevel, unsigned int, 0)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_log_file, logFile)
+DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_socket_backlog, socketBacklog, unsigned int, 0)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_file_descriptor_log_file, fileDescriptorLogFile)
 DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_max_pool_size, maxPoolSize, unsigned int, 1)
 DEFINE_SERVER_INT_CONFIG_SETTER(cmd_passenger_pool_idle_time, poolIdleTime, unsigned int, 0)
@@ -247,6 +303,8 @@ DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_default_user, defaultUser)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_default_group, defaultGroup)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_data_buffer_dir, dataBufferDir)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_instance_registry_dir, instanceRegistryDir)
+DEFINE_SERVER_BOOLEAN_CONFIG_SETTER(cmd_passenger_disable_security_update_check, disableSecurityUpdateCheck)
+DEFINE_SERVER_STR_CONFIG_SETTER(cmd_passenger_security_update_check_proxy, securityUpdateCheckProxy)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_union_station_gateway_address, unionStationGatewayAddress)
 DEFINE_SERVER_INT_CONFIG_SETTER(cmd_union_station_gateway_port, unionStationGatewayPort, int, 1)
 DEFINE_SERVER_STR_CONFIG_SETTER(cmd_union_station_gateway_cert, unionStationGatewayCert)
@@ -403,6 +461,11 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"Passenger log file."),
+	AP_INIT_TAKE1("PassengerSocketBacklog",
+		(Take1Func) cmd_passenger_socket_backlog,
+		NULL,
+		RSRC_CONF,
+		"Override size of the socket backlog."),
 	AP_INIT_TAKE1("PassengerFileDescriptorLogFile",
 		(Take1Func) cmd_passenger_file_descriptor_log_file,
 		NULL,
@@ -448,6 +511,16 @@ const command_rec passenger_commands[] = {
 		NULL,
 		RSRC_CONF,
 		"The directory to register the instance to."),
+	AP_INIT_FLAG("PassengerDisableSecurityUpdateCheck",
+		(FlagFunc) cmd_passenger_disable_security_update_check,
+		NULL,
+		RSRC_CONF,
+		"Whether to enable the security update check & notify."),
+	AP_INIT_TAKE1("PassengerSecurityUpdateCheckProxy",
+		(Take1Func) cmd_passenger_security_update_check_proxy,
+		NULL,
+		RSRC_CONF,
+		"Use specified http/SOCKS proxy for the security update check."),
 	AP_INIT_TAKE1("PassengerMaxPreloaderIdleTime",
 		(Take1Func) cmd_passenger_max_preloader_idle_time,
 		NULL,
